#include <node.h>
#include <iostream>
#include <unistd.h>
#include <uv.h>
#include <thread>
#include <sstream>
#include <functional>
#include <string>
#include <cameo/api/cameo.h>

using namespace std;
using namespace std::placeholders;

namespace nomad {

using v8::Function;
using v8::FunctionCallbackInfo;
using v8::Isolate;
using v8::Context;
using v8::Local;
using v8::Object;
using v8::String;
using v8::Value;
using v8::Number;
using v8::Integer;
using v8::Boolean;
using v8::Array;
using v8::Persistent;

unique_ptr<cameo::Server> server;
unique_ptr<cameo::Server> remoteServer;
unique_ptr<cameo::App> vexpSpy;
unique_ptr<cameo::coms::Requester> requester;
Isolate * v8Isolate;

string nomadEndpoint;
std::string VEXPSPY = "vexpspy";

/**
 * Init function to initialise the Cameo Nomad addon.
 */
void Init(const FunctionCallbackInfo<Value>& args) {

	cout << "initialising vEXP spy" << endl;

	// Get the V8 isolate.
	v8Isolate = args.GetIsolate();

	// Local endpoint.
	v8::String::Utf8Value param1(v8Isolate, args[0]);
	string localEndpoint(*param1);

    // Nomad endpoint.
	v8::String::Utf8Value param2(v8Isolate, args[1]);
	string nomadEndpoint(*param2);

	// Init the app if it is not already done.
	if (cameo::This::getId() == -1) {

		cameo::This::init("vEXP", localEndpoint);

		cout << "initialised cameo" << endl;
	}

    // Init nomad3d positions.
    server = cameo::Server::create(cameo::This::getServer().getEndpoint());
    server->init();

    cout << "cameo server " << *server << endl;

    vexpSpy = server->connect(VEXPSPY);
	if (vexpSpy) {
		// The application exists from a previous server session
		vexpSpy->kill();
		cameo::State state = vexpSpy->waitFor();
		cout << "terminated old vEXP spy with state " << cameo::toString(state) << endl;
	}

	vector<string> appArgs;
    appArgs.push_back(nomadEndpoint);

    vexpSpy = server->start(VEXPSPY, appArgs);

    cout << "vEXP spy " << *vexpSpy << endl;

    // Create the requester
	requester = cameo::coms::Requester::create(*vexpSpy, "get_data");

	if (!requester) {
		cout << "cannot create requester" << endl;
		return;
	}

	requester->init();

	// Create the remote server.
	remoteServer = cameo::Server::create(nomadEndpoint);
	remoteServer->init();

	args.GetReturnValue().Set(Undefined(v8Isolate));
}

void GetData(const FunctionCallbackInfo<Value>& args) {

	std::string reqData("DATA");

	// Send the request.
	requester->send(reqData);

	// Wait for the response.
	optional<string> response = requester->receive();
    
    args.GetReturnValue().Set(String::NewFromUtf8(args.GetIsolate(), response.value().c_str()).ToLocalChecked());
}

/**
 * The init function declares what we will make visible to node.
 */
void init(Local<Object> exports) {

	// Register the functions.
	NODE_SET_METHOD(exports, "init", Init);
	NODE_SET_METHOD(exports, "getData", GetData);
}

NODE_MODULE(addonvexpspy, init)

}
