const Controller = require('./controller');

class BzController extends Controller {
    constructor(bz, bzPanel) {
        super();
        this._bz = bz;

        this._bzPanel = bzPanel;
        PubSub.subscribe('change-sample', () => { this._bz.recreateGeometry(); });
        PubSub.subscribe('change-peaks', () => { this._bz.recreateGeometry(); });
        PubSub.subscribe('change-plane', () => { this._bz.recreateGeometry(); });
        
        PubSub.subscribe('update-path', (msg, value) => { this.updatePath(value); });
        
        PubSub.subscribe('refresh-all-bz-view', (msg, value) => { this.refreshAlBzView(value); })
        
        PubSub.subscribe('update-bz-struc', (msg, value) => { this.updateBzStruc(value); })

    }


    /**
     * This function will recreate geometry from scratch and set value
     * Only use for initiate
     */

    refreshAlBzView(value) {
        this.colorValueControler(value);
        this.writeValueOnBz3D(this._bz, value);
        this._bz.recreateGeometry();
    }

    /**
     * This function will change bz struc
     * @param {*} value 
     */
    updateBzStruc(value) {
        this.writeValueOnBz3D(this._bz, value);
        this._bz.changeBzStruct();
    }
    /**
     * This function will update offset position in bz-view
     * @param {*} value 
     */
    updatePosition(value) {
        this.writeValueOnBz3D(this._bz, value);
        this._bz.changePosition();
    }


    /**
     * This function will control that valu for RGB and alpha are not wrong
     * @param {*} value 
     */
    colorValueControler(value) {
        //cap face rgb between 0 and 255, use congruance if value not valide
        for (let index = 0; index < 3; index++) {
            if (value.faceColor[index] > 255) {
                value.faceColor[index] = value.faceColor[index] % 255;
            }
            if (value.faceColor[index] < 0) {
                value.faceColor[index] = 255 - value.faceColor[index] % 255
            }
        }
        //cap alpha of face color
        if (value.faceColor[3] > 1) {
            value.faceColor[3] = 1
        }
        if (value.faceColor[3] < 0) {
            value.faceColor[3] = 0
        }
        //cap edge rgb between 0 and 255, use congruance if value not valide
        for (let index = 0; index < 3; index++) {
            if (value.edgeColor[index] > 255) {
                value.edgeColor[index] = value.edgeColor[index] % 255;
            }
            if (value.edgeColor[index] < 0) {
                value.edgeColor[index] = 255 - value.edgeColor[index] % 255
            }
        }
        //cap alpha of edge color
        if (value.edgeColor[3] > 1) {
            value.edgeColor[3] = 1
        }
        if (value.edgeColor[3] < 0) {
            value.edgeColor[3] = 0
        }
    }

    /**
     * This function wil chaneg takin path
     * @param {*} value 
     */
    updatePath(value) {
        if (value.takinPathValide == false) {
            value.bzViewMode = "hide";
            this.updateBzView(value);
        }
        else {
            bz._takinPath = value.takinPath;
            this.updateBzView(value);
        }
    }



    /**
     * This function will write on bz all data
     * @param {*} bz 
     * @param {*} value 
     */
    writeValueOnBz3D(bz, value) {
        bz._colorTemplate = value.faceTemplate;
        bz._bzViewMode = value.bzViewMode;
        bz._showFace = value.faceTemplate != "hide";
        bz._normalColorMode = value.normalPerspective;
        bz._showEdge = value.showEdge;
        bz._faceColor = value.faceColor;
        bz._edgeColor = value.edgeColor;
        bz._offsetPosition = value.offsetPosition;
    }



    /**
     * This function will request to bz view to recreate from scratch
     * @param {*} value 
     */
    updateBzView(value) {
        this.colorValueControler(value);
        this._bzPanel.setInput(value);
        this.writeValueOnBz3D(this._bz, value);
        this._bz.recreateGeometry();
    }



}
module.exports = BzController;
