const Controller = require('./controller');
const { requester } = require('../main/context');
const { peaks } = require('../server-state');

class PeaksController extends Controller {
    constructor(peaksPanel) {
        super();

        this._peaksPanel = peaksPanel;

        this.init();

        // Action handlers.
        PubSub.subscribe('update-peaks', (msg, value) => { this.updatePeaks(value); });
        PubSub.subscribe('change-peaks', () => { this.updatePanel(); });
    }

    init() {
        this._peaksPanel.setInputs(peaks);
    }

    /**
     * Update peaks panel with model data.
     */
    updatePanel() {
        this._peaksPanel.setInputs(peaks);
    }

    /**
     * Update plane bounds (bragg peaks).
     */
    updatePeaks(peaksParam) {
        requester.request('setPeaks', peaksParam, (data) => {
            // Update local peaks state.
            peaks.update(data);

            PubSub.publish('change-peaks', 0);
        });
    }
}

module.exports = PeaksController;