const viewManager = require('../../utils/view-manager');
const PubSub = require('pubsub-js');
const { grid } = require('../../server-state');
const { menuEvent } = require('../../main/context');

class GridPanelView {

    constructor() {

        let html = `
        <div class='container' style='width:100%'>
            <div class='row'>
                <label class='col-sm-4' for='gridSize'>Subdivision</label>
                <input class='col-sm-4' type='text' name='gridSize' id='gridSize' disabled />
                <div class='col-sm-4'/>
            </div>
            <div class='row' style='display: none;'>
                <label class='col-sm-4' for='gridDelta'>Precision</label>
                <input class='col-sm-4' type='text' name='gridDelta' id='gridDelta' disabled />
                <div class='col-sm-4'/>
            </div>
            <div class='row'>
                <label class='col-sm-4' for='useAdaptative'>Adaptative</label>
                <input class='col-md-4' type='checkbox' name='useAdaptative' id='useAdaptative' />
                <div class='col-sm-4'/>
            </div>
        </div>`;

        // Using lambda function does not work here.
        myLayout.registerComponent('Grid Settings', function(container, state) {
            container.getElement().html(html);
        });

        viewManager.addView(this);
    }

    init() {

        // Set the adaptative disabled as the 3D view is first active.
        $('#useAdaptative').prop('disabled', true);

        // Disable the adaptative when toggling the view.
        menuEvent.on('toggle-2d-view', (event, value) => {
            $('#useAdaptative').prop('disabled', !value);
        });

        $("#gridSize, #gridDelta").keypress((event) => {
            if (event.which == 13 || event.keyCode == 13 ) {
                PubSub.publish('update-grid', this.getInputs());
            }
        });

        $('#useAdaptative').change((event) => {
            if(event.target.checked) {
                $("#gridSize, #gridDelta").prop('disabled', true);
            } else {
                $("#gridSize, #gridDelta").prop('disabled', false);
            }

            PubSub.publish('update-grid', this.getInputs()); 
        });

        // Initialize inputs from model data.
        this.setInputs({ numDiv: grid.numDiv, delta: grid.delta, adaptative: grid.adaptative });
    }

    /**
     * Set grid inputs value.
     */
    setInputs(value) {
        $('#gridSize').val(value.numDiv);
        $('#gridDelta').val(value.delta);

        if (value.adaptative) {
            $('#useAdaptative').prop('checked', true);
            $("#gridSize, #gridDelta").prop('disabled', true);
        } else {
            $('#useAdaptative').prop('checked', false);
            $("#gridSize, #gridDelta").prop('disabled', false);
        }
    }

    /**
     * Get grid inputs and store them into an object.
     */
    getInputs() {
        return {
            numDiv: parseInt($("#gridSize").val()),
            delta: parseFloat($("#gridDelta").val()),
            adaptative: $('#useAdaptative').prop('checked')
        };
    }

    render() {
    }
}

module.exports = GridPanelView;