const viewManager = require('../../utils/view-manager');
const PubSub = require('pubsub-js');
const { peaks } = require('../../server-state');

class PeaksPanelView {

    constructor() {

        let html = `
        <div class='container' style='width:100%'>
            <div class='row'>
                <div class='col-sm-3'>Lower</div>
                <input type='text' class='col-sm-3 border-right-text-input' id='lowX'/>
                <input type='text' class='col-sm-3 border-right-text-input border-left-text-input' id='lowY'/>
                <input type='text' class='col-sm-3 border-left-text-input' id='lowZ'/>
            </div>
            <div class='row'>
                <div class='col-sm-3'>Upper</div>
                <input type='text' class='col-sm-3 border-right-text-input ' id='upperX'/>
                <input type='text' class='col-sm-3 border-right-text-input border-left-text-input' id='upperY'/>
                <input type='text' class='col-sm-3 border-left-text-input' id='upperZ'/>
            </div>
            <div class='row'>
                <div class='col-sm-3'>Opacity</div>
                <input type='text' class='col-sm-3' id='opacity'/>
            </div>
        </div>`;

        // Using lambda function does not work here.
        myLayout.registerComponent('Bragg Peaks', function(container, state) {
            container.getElement().html(html);
        });

        viewManager.addView(this);
    }

    init() {

        // When the view is initialised, we can access to the html elements.
        $("#lowX, #lowY, #lowZ, #upperX, #upperY, #upperZ, #opacity").keypress((event) => {

            if (event.which == 13 || event.keyCode == 13) {
                PubSub.publish('update-peaks', this.getInputs());
            }
        });

        // Initialize inputs from model data.
        this.setInputs(peaks);
    }

    /**
     * Set peaks inputs value.
     */
    setInputs(value) {
        $('#lowX').val(value.lowX);
        $('#lowY').val(value.lowY);
        $('#lowZ').val(value.lowZ);
        $('#upperX').val(value.upperX);
        $('#upperY').val(value.upperY);
        $('#upperZ').val(value.upperZ);
        $('#opacity').val(value.sphereOpacity);
    }

    /**
     * Get peaks inputs value and store them into an object.
     */
    getInputs() {
        return {
            lowX: parseInt($('#lowX').val()),
            lowY: parseInt($('#lowY').val()),
            lowZ: parseInt($('#lowZ').val()),
            upperX: parseInt($('#upperX').val()),
            upperY: parseInt($('#upperY').val()),
            upperZ: parseInt($('#upperZ').val()),
            sphereOpacity: parseFloat($('#opacity').val()),
        }
    }

    render() {
    }
}

module.exports = PeaksPanelView;
