const Receiver = require('../receiver');

class WebReceiver extends Receiver {
    constructor(wss) {
        super();

        this._wss = wss;
        this._listeners = {};

        this._wss.addEventListener('message', (event) => {
            let packet = JSON.parse(event.data);
            
            if(this._listeners[packet.event]) {
                for(let listener of this._listeners[packet.event]) {
                    listener(packet, packet.data);
                }
            }
        });

        this._wss.addEventListener('open', (event) => {
            console.log('receiver socket opened');
        });

        this._wss.addEventListener('close', (event) => {
            console.log('receiver socket closed');
        });

        this._wss.addEventListener('error', (event) => {
            console.log('receiver socket error');
        });
    }

    on(event, callback) {

        if(this._listeners[event]) {
            this._listeners[event].push(callback);
        } else {
            this._listeners[event] = [ callback ];
        }
    }
}

module.exports = WebReceiver;