const { app, BrowserWindow } = require('electron');
const path = require('path');
const url = require('url');
const { receiver, sender } = require('../context');
const PubSub = require('pubsub-js');

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let win;

global.events = {events: {}};
global.toggleID = {toggle: false};

function createWindow() {

    // Create the browser window.
    win = new BrowserWindow({
        width: 875,
        height: 600,
        
        icon: 'assets/icons/png/64x64.png',
		webPreferences: {
			// Does not work if set to true.
			contextIsolation: false,
			webSecurity: true,
            webviewTag:true,
            nativeWindowOpen: false,
            enableRemoteModule: true,
			// Added to have 'require' keyword.
			nodeIntegration: true
		}
    });

    sender.win = win;
    receiver.win = win;

    // Load the index.html of the app.
    win.loadURL(url.format({
        pathname: path.join(__dirname, './view/index.html'),
        protocol: 'file:',
        slashes: true
    }));

    // Load the menu.
    require('./main-menu');
    //let receiver = new ElectronReceiver(win);

    win.on('closed', () => {
        if (process.argv[2] === "--nomad") {
            let nomadAccessor = require("../../../build/Release/addoncameonomad");
            nomadAccessor.terminate();
        }

        win = null;
    });
}

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.on('ready', createWindow);

// Quit when all windows are closed.
app.on('window-all-closed', () => {
    // On macOS it is common for applications and their menu bar
    // to stay active until the user quits explicitly with Cmd + Q
    if (process.platform !== 'darwin') {
        app.quit()
    }
});

app.on('activate', () => {
    // On macOS it's common to re-create a window in the app when the
    // dock icon is clicked and there are no other windows open.
    if (win === null) {
        createWindow();
    }
});
