const Model = require('./model');
const sqwdata = require('./sqw/sqw-data');
const sqwformula = require('./sqw/sqw-formula');

class ExpDataModel extends Model {
    constructor(sqwtype, texformula, texwidth, texheight, texminsqw, texmaxsqw, texE, texESigma) {
        super();

        this._sqwtype = sqwtype;
        this._texformula = texformula;
        this._texwidth = texwidth;
        this._texheight = texheight;
        this._texminsqw = texminsqw;
        this._texmaxsqw = texmaxsqw;
        this._texE = texE;
        this._texESigma = texESigma;
        this._filename = null;

        this.initSqwData();
    }

    // Getters.
    get sqwtype() { return this._sqwtype; }
    get texformula() { return this._texformula; }
    get texwidth() { return this._texwidth; }
    get texheight() { return this._texheight; }
    get texminsqw() { return this._texminsqw; }
    get texmaxsqw() { return this._texmaxsqw; }
    get texE() { return this._texE; }
    get texESigma() { return this._texESigma; }
    get filename() { return this._filename; }

    // Setters.
    set sqwtype(value) { 
        if(value === 'sqw-formula' || value === 'sqw-none') {
            this._filename = null;
        }
        this._sqwtype = value;
    }

    set filename(value) { 
        this._filename = value; 
    }

    set texformula(value) { 
        this._texformula = value;
    }

    set texwidth(value) { 
        this._texwidth = value;
    }

    set texheight(value) { 
        this._texheight = value;
    }

    set texminsqw(value) { 
        this._texminsqw = value;
    }

    set texmaxsqw(value) { 
        this._texmaxsqw = value;
    }

    set texE(value) { 
        this._texE = value;
    }

    set texESigma(value) {
        this._texESigma = value;
    }

    /**
     * Set all experimental data.
     */
    setExpData(sqwtype, texformula, texwidth, texheight, texminsqw, texmaxsqw, texE, texESigma, filename = null, filedata = null) {
        this._sqwtype = sqwtype;
        this._texformula = texformula;
        this._texwidth = texwidth;
        this._texheight = texheight;
        this._texminsqw = texminsqw;
        this._texmaxsqw = texmaxsqw;
        this._texE = texE;
        this._texESigma = texESigma;
        this._filename = filename;

        this.initSqwData(filedata);     
    }

    initSqwData(filedata) {

        let options = {
            sqwtype: this._sqwtype,
            formula: this._texformula,
            sigmaE: this._texESigma,
            minS: this._texminsqw,
            maxS: this._texmaxsqw,
            filedata
        };

        if (this._filename && this._filename !== null) {
            options.filename = this._filename;
        }

        switch (options.sqwtype) {
            case "sqw-none":
                this._sqwData = null;
                break;
            case "sqw-data":
                this._sqwData = new sqwdata.Sqw(options);
                break;
            case "sqw-formula":
                this._sqwData = new sqwformula.Sqw(options);
                break;
            default:
                console.error("Invalid sqw type selected.");
        }
    }

    query(h, k, l, E) {

        return this._sqwData.query(h, k, l, E);
    }

    toObject() {
        return {
            sqwtype: this._sqwtype,
            texformula: this._texformula,
            texwidth: this._texwidth,
            texheight: this._texheight,
            texESigma: this._texESigma,
            texminsqw: this._texminsqw,
            texmaxsqw: this._texmaxsqw,
            texE: this._texE,
            filename: this._filename
        };
    }

    save(saver) {
        saver.expdata = this.toObject();
    }

    restore(saver) {
        let expdata = saver.expdata;

        this.setExpData(
            expdata.sqwtype, expdata.texformula, 
            expdata.texwidth, expdata.texheight, 
            expdata.texminsqw, expdata.texmaxsqw, 
            expdata.texE, expdata.texESigma, expdata.filename
        );
    }
}

module.exports = ExpDataModel;