/**
 * S(Q,E) dummy S(q,w) for testing
 * @author Tobias Weber <tweber@ill.fr>
 * @date Jan-2018
 * @license see "LICENSE" file
 */

const math = require("mathjs");
const sqwdata = require("./sqw-data");


/**
 * Euklidian metric
 */
function metric(pt1, pt2)
{
    const vec1 = math.matrix([ pt1.coordinates[0], pt1.coordinates[1], pt1.coordinates[2], pt1.coordinates[3] ]);
    const vec2 = math.matrix([ pt2.coordinates[0], pt2.coordinates[1], pt2.coordinates[2], pt2.coordinates[3] ]);

    const vecdiff = math.subtract(vec2, vec1);
    return math.sqrt(math.dot(vecdiff, vecdiff));
};


class SqwDummy
{
    constructor(options = {})
    { }


    /**
     * Gaussian intensity at Bragg peaks
     */
    query(h, k, l, E)
    {
        const vechklE = { "coordinates" : [h,k,l,E] };
        const vecBragg = { "coordinates" : [math.round(h), math.round(k), math.round(l), math.round(E)] };

        const q = metric(vechklE, vecBragg);
        const S = math.exp(-q*q/0.05);

        return { "S" : S, "coordinates" : vecBragg.coordinates };
    }
};



// ----------------------------------------------------------------------------
/**
 * exports
 */
module.exports = {
    // class
    "Sqw" : SqwDummy,

    // helper functions
    "mapColor" : sqwdata.mapColor,
};
// ----------------------------------------------------------------------------
