const Sample = require('./model/samples/sample');
const config = require('./config');

/*
 * Class implementing the calculations by a Nomad access.
 */
class NomadDataAccessor {

    constructor(id) {
        this._nomadAccessor = require("../../build/Release/addoncameonomad");
        try {
            if (id == -1) {
                this._nomadAccessor.init(config.localEndpoint, config.nomadEndpoint);
            }
            else {
                this._nomadAccessor.init(config.localEndpoint, config.nomadEndpoint, id);
            }

        } catch (e) {
            console.error(e);
        }

        // Nomad property ids.
        this._a1PropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'calca1');
        this._a2PropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'calca2');
        this._a3PropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'calca3');
        this._a3pPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'calca3p');
        this._a4PropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'calca4');
        this._a5PropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'calca5');
        this._a6PropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'calca6');

        // ki & kf values.
        this._kiPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'ki');
        this._kfPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'kf');

        // Scattering sense.
        this._ssPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'ss');

        // Calculated values property.
        this._calcValuesPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'calcvalues');

        // U an B matrices and can be used to calculate UB matrix.
        this._uMatrixPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'Umatrix');
        this._bMatrixPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'Bmatrix');

        // Q Values.
        this._qhPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'qh');
        this._qkPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'qk');
        this._qlPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'ql');
        this._qmPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'qm');
        this._fxPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'fx');

        this._asPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'as');
        this._bsPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'bs');
        this._csPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'cs');
        this._aaPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'aa');
        this._bbPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'bb');
        this._ccPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'cc');

        this._syncSample = true;

        // A&B Plane properties.
        this._axPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'ax');
        this._ayPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'ay');
        this._azPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'az');
        this._bxPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'bx');
        this._byPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'by');
        this._bzPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'bz');

        // Calculation status and error.
        this._calculationStatusPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'calculation_status');
        this._calculationErrorPropertyId = this._nomadAccessor.getPropertyId('VEXPController', 'calculation_error');

        // XBU path.
        this._xbuPathPropertyId = this._nomadAccessor.getPropertyId('Title', 'xbupath');

        // Resync with Nomad.
        this.resync();

        // Callbacks
        this._cb_U_changed = function (val) { };
        this._cb_B_changed = function (val) { };

        // Initialize U and B.
        this.recalcUB();
    }

    toNumber(value) {
        if (!isFinite(value) || isNaN(value)) {
            return 0.0;
        }
        return value;
    }

    resync() {
       
        this._ax = this.toNumber(this._nomadAccessor.getFloat64Property(this._axPropertyId));
        this._ay = this.toNumber(this._nomadAccessor.getFloat64Property(this._ayPropertyId));
        this._az = this.toNumber(this._nomadAccessor.getFloat64Property(this._azPropertyId));
        this._bx = this.toNumber(this._nomadAccessor.getFloat64Property(this._bxPropertyId));
        this._by = this.toNumber(this._nomadAccessor.getFloat64Property(this._byPropertyId));
        this._bz = this.toNumber(this._nomadAccessor.getFloat64Property(this._bzPropertyId));

        this._as = this.toNumber(this._nomadAccessor.getFloat64Property(this._asPropertyId));
        this._bs = this.toNumber(this._nomadAccessor.getFloat64Property(this._bsPropertyId));
        this._cs = this.toNumber(this._nomadAccessor.getFloat64Property(this._csPropertyId));
        this._aa = this.toNumber(this._nomadAccessor.getFloat64Property(this._aaPropertyId));
        this._bb = this.toNumber(this._nomadAccessor.getFloat64Property(this._bbPropertyId));
        this._cc = this.toNumber(this._nomadAccessor.getFloat64Property(this._ccPropertyId));

        this._ki = this.toNumber(this._nomadAccessor.getFloat64Property(this._kiPropertyId));
        this._kf = this.toNumber(this._nomadAccessor.getFloat64Property(this._kfPropertyId));
        this._q_h = this.toNumber(this._nomadAccessor.getFloat64Property(this._qhPropertyId));
        this._q_k = this.toNumber(this._nomadAccessor.getFloat64Property(this._qkPropertyId));
        this._q_l = this.toNumber(this._nomadAccessor.getFloat64Property(this._qlPropertyId));
        this._fx = this.toNumber(this._nomadAccessor.getInt32Property(this._fxPropertyId));
        this._ss = this.toNumber(this._nomadAccessor.getInt32Property(this._ssPropertyId));

        this.B = [];
        this.U = [];
        this._UB_obsolete = true;
    }

    resyncSpy(data) {

        // Set the data.
        if (this._syncSample) {
            this._as = data.as;
            this._bs = data.bs;
            this._cs = data.cs;
            this._aa = data.aa;
            this._bb = data.bb;
            this._cc = data.cc;
        }
        this._ax = data.ax;
        this._ay = data.ay;
        this._az = data.az;
        this._bx = data.bx;
        this._by = data.by;
        this._bz = data.bz;
        this._q_h = data.qh;
        this._q_k = data.qk;
        this._q_l = data.ql;
        this._en = data.en;
        this._qm = data.qm;

        this._fx = data.fx;
        this._ss = data.ss;

        this._ki = data.ki;
        this._kf = data.kf;

        if (data.dTypeScan === "center") {

            let n = (data.nbPointsScan - 1) / 2;

            this._qhScan = data.qhScan - n * data.dqhScan;
            this._qkScan = data.qkScan - n * data.dqkScan;
            this._qlScan = data.qlScan - n * data.dqlScan;
            this._enScan = data.enScan - n * data.denScan;
        }
        else if (data.dTypeScan === "min") {
            this._qhScan = data.qhScan;
            this._qkScan = data.qkScan;
            this._qlScan = data.qlScan;
            this._enScan = data.enScan;
        }

        this._dqhScan = data.dqhScan;
        this._dqkScan = data.dqkScan;
        this._dqlScan = data.dqlScan;
        this._denScan = data.denScan;

        this._nbPointsScan = data.nbPointsScan;
        this._runningScan = data.runningScan;

        this._kiScan = data.kiScan;
        this._kfScan = data.kfScan;

        this._fxScan = data.fxScan;
        this._ssScan = data.ssScan;
      
        this._nameScan = data.nameScan.toString();
        this._startingTime = data.startingTime;

        this.recalcUB();
    }

    recalcUB() {
        let sample = new Sample("", "", "none",
            this._as, this._bs, this._cs,
            this._aa, this._bb, this._cc,
            [this._ax, this._ay, this._az],
            [this._bx, this._by, this._bz]
        );

        sample.calculateReciprocals();
        sample.calculateUBMatrix();
        this.U = sample.u_matrix.elements;
        this.B = sample.b_matrix.elements;

        this._UB_obsolete = false;
    }

    get xbuPath() { return this._nomadAccessor.getStringProperty(this._xbuPathPropertyId); }

    set q_h(value) { this._q_h = value; }
    get q_h() { return this._q_h; }
    set q_k(value) { this._q_k = value; }
    get q_k() { return this._q_k; }
    set q_l(value) { this._q_l = value; }
    get q_l() { return this._q_l; }
    set fx(value) { this._fx = value; }
    get fx() { return this._fx; }

    get bMatrix() {
        if (this._UB_obsolete)
            this.recalcUB();
        return this.B;
    }
    
    get uMatrix() {
        if (this._UB_obsolete)
            this.recalcUB();
        return this.U;
    }

    set as(value) { this._as = value; this._UB_obsolete = true; }
    get as() { return this._as; }
    set bs(value) { this._bs = value; this._UB_obsolete = true; }
    get bs() { return this._bs; }
    set cs(value) { this._cs = value; this._UB_obsolete = true; }
    get cs() { return this._cs; }
    set aa(value) { this._aa = value; this._UB_obsolete = true; }
    get aa() { return this._aa; }
    set bb(value) { this._bb = value; this._UB_obsolete = true; }
    get bb() { return this._bb; }
    set cc(value) { this._cc = value; this._UB_obsolete = true; }
    get cc() { return this._cc; }

    set syncSample(value) {
        this._syncSample = value;
    }

    set ax(value) { this._ax = value; this._UB_obsolete = true; }
    get ax() { return this._ax; }
    set ay(value) { this._ay = value; this._UB_obsolete = true; }
    get ay() { return this._ay; }
    set az(value) { this._az = value; this._UB_obsolete = true; }
    get az() { return this._az; }
    set bx(value) { this._bx = value; this._UB_obsolete = true; }
    get bx() { return this._bx; }
    set by(value) { this._by = value; this._UB_obsolete = true; }
    get by() { return this._by; }
    set bz(value) { this._bz = value; this._UB_obsolete = true; }
    get bz() { return this._bz; }

    set ki(value) { this._ki = value; }
    get ki() { return this._ki; }

    set kf(value) { this._kf = value; }
    get kf() { return this._kf; }

    get ss() { return this._ss; }

    set calcValues(value) {
        this._nomadAccessor.setStringProperty(this._calcValuesPropertyId, value);
    }

    get a1() {
        return this._nomadAccessor.getFloat64Property(this._a1PropertyId);
    }

    get a2() {
        return this._nomadAccessor.getFloat64Property(this._a2PropertyId);
    }

    get a3() {
        return this._nomadAccessor.getFloat64Property(this._a3PropertyId);
    }

    get a3p() {
        return this._nomadAccessor.getFloat64Property(this._a3pPropertyId);
    }

    get a4() {
        return this._nomadAccessor.getFloat64Property(this._a4PropertyId);
    }

    get a5() {
        return this._nomadAccessor.getFloat64Property(this._a5PropertyId);
    }

    get a6() {
        return this._nomadAccessor.getFloat64Property(this._a6PropertyId);
    }

    get calculationStatus() {
        return this._nomadAccessor.getInt32Property(this._calculationStatusPropertyId);
    }

    get calculationError() {
        return this._nomadAccessor.getInt32Property(this._calculationErrorPropertyId);
    }

    get qhScan() {
        return this._qhScan;
    }

    get qkScan() {
        return this._qkScan;
    }

    get qlScan() {
        return this._qlScan;
    }

    get enScan() {
        return this._enScan;
    }

    get dqhScan() {
        return this._dqhScan;
    }

    get dqkScan() {
        return this._dqkScan;
    }

    get dqlScan() {
        return this._dqlScan;
    }

    get denScan() {
        return this._denScan;
    }

    get kiScan() {
        return this._kiScan;
    }

    get kfScan() {
        return this._kfScan;
    }

    get fxScan() {
        return this._fxScan;
    }

    get ssScan() {
        return this._ssScan;
    }

    get nbPointsScan() {
        return this._nbPointsScan;
    }

    get runningScan() {
        return this._runningScan;
    }

    get nameScan() {
        return this._nameScan;
    }

    get startingTime() {
        return this._startingTime;
    }
    
}

// We export the constructor function.
module.exports = NomadDataAccessor;
